@@ -1,6 +1,6 @@
 ;;; ada-core.el --- core facilities for ada-mode -*- lexical-binding:t -*-
 
-;; Copyright (C) 1994, 1995, 1997 - 2017, 2019 - 2020  Free Software Foundation, Inc.
+;; Copyright (C) 1994, 1995, 1997 - 2017, 2019 - 2021  Free Software Foundation, Inc.
 ;;
 ;; Author: Stephen Leake <stephen_leake@member.fsf.org>
 ;; Maintainer: Stephen Leake <stephen_leake@member.fsf.org>
@@ -63,7 +63,11 @@ Called by `syntax-propertize', which is called by font-lock in
   "Goto start of declarative region containing point."
   (interactive)
   (wisi-validate-cache (point-min) (point-max) t 'navigate)
-
+  (push-mark)
+  (when (looking-back "declare" (line-beginning-position))
+    ;; We just did ada-goto-declarative-region-start to get here; we
+    ;; want the next one up.
+    (backward-word 1))
   (let ((done nil)
 	(start-pos (point))
 	(outermost nil)
@@ -84,14 +88,16 @@ Called by `syntax-propertize', which is called by font-lock in
 
 	(if (ada-declarative-region-start-p cache)
 	    (if (< (point) start-pos)
+		;; found it.
 		(progn
-		  (forward-word);; past 'is'
+		  (forward-word);; past 'is' or 'declare'.
 		  (setq done t))
 
 	      ;; test/ada_mode-nominal.adb function F2
 	      ;;
-	      ;; start-point is in a formal_part or aspect_clause; we
-	      ;; want the next level up.
+	      ;; start-point is in a subprogram_declarator,
+	      ;; formal_part, aspect_clause, etc; code that contains a
+	      ;; declarative part. We want the next level up.
 	      (if outermost
 		  ;; there is no next level up; add the use_clause in the context_clause.
 		  (progn
@@ -184,9 +190,15 @@ is the package spec.")
 (defconst ada-refactor-format-paramlist 5)
 
 (defun ada-refactor (action)
+  "Perform refactor action ACTION on symbol at point."
   (wisi-validate-cache (line-end-position -7) (line-end-position 7) t 'navigate)
   (save-excursion
+    ;; We include punctuation and quote for operators.
     (skip-syntax-backward "w_.\"")
+
+    ;; Skip leading punctuation, for "-Foo.Bar".
+    (skip-syntax-forward ".")
+
     (let* ((edit-begin (point))
 	   (cache (wisi-goto-statement-start))
 	   (parse-begin (point))
@@ -706,10 +718,10 @@ Deselects the current project first."
 (make-obsolete 'ada-select-prj-file 'wisi-prj-select-cache "ada-mode 7.0")
 
 (cl-defgeneric ada-prj-select-compiler (compiler project)
-  "PROJECT has been selected; set any project options that are both Ada and compiler specific.")
+  "Set PROJECT options that are Ada and compiler specific.")
 
 (cl-defgeneric ada-prj-deselect-compiler (compiler project)
-  "PROJECT has been deselected; unset any project options that are both Ada and compiler specific.")
+  "Unset any PROJECT options that are both Ada and compiler specific.")
 
 (cl-defmethod wisi-prj-select :after ((project ada-prj))
   (ada-prj-select-compiler (ada-prj-compiler project) project))
