@@ -1,13 +1,13 @@
 ;;; ada-mode.el --- major-mode for editing Ada sources  -*- lexical-binding:t -*-
 ;;
-;; Copyright (C) 1994, 1995, 1997 - 2020  Free Software Foundation, Inc.
+;; Copyright (C) 1994, 1995, 1997 - 2021  Free Software Foundation, Inc.
 ;;
 ;; Author: Stephen Leake <stephen_leake@stephe-leake.org>
 ;; Maintainer: Stephen Leake <stephen_leake@stephe-leake.org>
 ;; Keywords: languages
 ;;  ada
-;; Version: 7.1.0
-;; package-requires: ((uniquify-files "1.0.1") (wisi "3.1.0") (emacs "25.0"))
+;; Version: 7.2.0
+;; package-requires: ((uniquify-files "1.0.1") (wisi "3.1.5") (emacs "25.3"))
 ;; url: http://www.nongnu.org/ada-mode/
 ;;
 ;; This file is part of GNU Emacs.
@@ -117,7 +117,7 @@
 (defun ada-mode-version ()
   "Return Ada mode version."
   (interactive)
-  (let ((version-string "7.1.0"))
+  (let ((version-string "7.1.8"))
     (if (called-interactively-p 'interactive)
 	(message version-string)
       version-string)))
@@ -144,7 +144,7 @@ rather than to the same column."
   :safe #'booleanp)
 
 (defcustom ada-which-func-parse-size 30000
-  "Minimum size of the region surrounding point that is parsed for `which-function-mode'."
+  "Minimum size of region around point parsed for `which-function-mode'."
   :group 'ada
   :type 'integer
   :safe #'integerp)
@@ -520,6 +520,37 @@ The extensions should include a `.' if needed.")
   "Alist used by `find-file' to find the name of the other package.
 See `ff-other-file-alist'.")
 
+(defconst ada-declaration-nonterms
+  '(
+    abstract_subprogram_declaration
+    entry_body
+    entry_declaration
+    expression_function_declaration
+    full_type_declaration
+    generic_instantiation
+    generic_package_declaration
+    generic_subprogram_declaration
+    null_procedure_declaration
+    object_declaration
+    package_body
+    package_declaration
+    pragma_g
+    private_extension_declaration
+    private_type_declaration
+    protected_body
+    protected_type_declaration
+    single_protected_declaration
+    single_task_declaration
+    subprogram_body
+    subprogram_declaration
+    subprogram_renaming_declaration
+    subtype_declaration
+    task_body
+    task_type_declaration
+    use_clause
+    )
+  "wisi-cache nonterminal symbols that are Ada declarations.")
+
 (defconst ada-parent-name-regexp
   "\\([[:alnum:]_\\.]+\\)\\.[[:alnum:]_]+"
   "Regexp for extracting the parent name from fully-qualified name.")
@@ -585,6 +616,7 @@ See `ff-other-file-alist'.")
 (defun ada-which-function (&optional include-type)
   "Return name of subprogram/task/package containing point.
 Also sets ff-function-name for ff-pre-load-hook."
+  (interactive) ;; because which-function-mode does not provide which-function to call intermittently!
   ;; Fail gracefully and silently, since this could be called from
   ;; which-function-mode.
   (let ((parse-begin (max (point-min) (- (point) (/ ada-which-func-parse-size 2))))
@@ -648,6 +680,8 @@ Also sets ff-function-name for ff-pre-load-hook."
 		    (task_body
 		     (setq result (ada-which-function-1 "task" nil)))
 		    ))
+		(when (called-interactively-p 'interactive)
+		  (message result))
 		result)))
 	(error "")))
     ))
@@ -818,13 +852,19 @@ compiler-specific compilation filters."
   ;; doesn't change, at least on Windows.
   (let ((start-buffer (current-buffer))
 	pos item file)
-    ;; We use `pop-to-buffer', not `set-buffer', so `forward-line'
-    ;; works. But that might eat an `other-frame-window-mode' prefix,
-    ;; which the user means to apply to ’ada-goto-source’ below;
-    ;; disable that temporarily.
+    ;; We use `pop-to-buffer', not `set-buffer', so point is correct
+    ;; for the current window showing compilation-last-buffer, and
+    ;; moving point in that window works. But that might eat an
+    ;; `other-frame-window-mode' prefix, which the user means to apply
+    ;; to ’ada-goto-source’ below; disable that temporarily.
     (let ((display-buffer-overriding-action nil))
       (pop-to-buffer compilation-last-buffer nil t)
       (setq pos (next-single-property-change (point) 'ada-secondary-error))
+      (unless pos
+	;; probably at end of compilation-buffer, in new compile
+	(goto-char (point-min))
+	(setq pos (next-single-property-change (point) 'ada-secondary-error)))
+
       (when pos
 	(setq item (get-text-property pos 'ada-secondary-error))
 	;; file-relative-name handles absolute Windows paths from
@@ -834,8 +874,7 @@ compiler-specific compilation filters."
 
 	;; Set point in compilation buffer past this secondary error, so
 	;; user can easily go to the next one.
-	(goto-char pos)
-	(forward-line 1))
+	(goto-char (next-single-property-change (1+ pos) 'ada-secondary-error)))
 
       (pop-to-buffer start-buffer nil t);; for windowing history
       )
@@ -850,10 +889,8 @@ compiler-specific compilation filters."
 (defun ada-goto-declaration-start-1 (include-type)
   "Subroutine of `ada-goto-declaration-start'."
   (let ((start (point))
-	(cache (wisi-get-cache (point)))
+	(cache (or (wisi-get-cache (point)) (wisi-backward-cache)))
 	(done nil))
-    (unless cache
-      (setq cache (wisi-backward-cache)))
     ;; cache is null at bob
     (while (not done)
       (if cache
@@ -888,7 +925,7 @@ compiler-specific compilation filters."
 		     (eq (wisi-cache-token cache) 'TASK))
 
 		    ))
-	    (unless (< start (wisi-cache-end cache))
+	    (unless (<= start (wisi-cache-end cache))
 	      ;; found declaration does not include start; find containing one.
 	      (setq done nil))
 	    (unless done
@@ -1120,13 +1157,13 @@ Must match wisi-ada.ads Language_Protocol_Version.")
 	  ))
 
 (defconst ada-wisi-named-begin-regexp
-  "\\bfunction\\b\\|\\bpackage\\b\\|\\bprocedure\\b\\|\\btask\\b"
+  "\\_<function\\_>\\|\\_<package\\_>\\|\\_<procedure\\_>\\|\\_<task\\_>"
   )
 
 (defconst ada-wisi-partial-begin-regexp
-  (concat "\\bbegin\\b\\|\\bdeclare\\b\\|"
+  (concat "\\_<begin\\_>\\|\\_<declare\\_>\\|"
 	  ada-wisi-named-begin-regexp
-	  "\\|\\bend;\\|\\bend " ada-name-regexp ";"))
+	  "\\|\\_<end;\\|\\_<end " ada-name-regexp ";"))
 
 (defconst ada-wisi-partial-end-regexp
   (concat ada-wisi-partial-begin-regexp
@@ -1149,13 +1186,18 @@ Must match wisi-ada.ads Language_Protocol_Version.")
   ;; begin
   ;;    Foo;
   ;;
-  ;; Inserting new line after 'Foo;'; if we include 'begin', there
-  ;; is no error (begin starts a statement), and the indent is
-  ;; computed incorrectly.
+  ;; Inserting new line after 'Foo;'; if we include 'begin' but not
+  ;; 'end;', there is no error (begin starts a statement), and the
+  ;; indent is computed incorrectly, because it is assumed that the
+  ;; line containing 'end;' is indented correctly.
   ;;
   ;; This is handled by the set of keywords in
   ;; ada-wisi-partial-begin-regexp.
   (cond
+   ((looking-at "[ \t]*\\_<begin\\_>")
+    ;; indenting 'begin'; best option is to assume it is indented properly
+    (point))
+
    ((wisi-search-backward-skip
      ada-wisi-partial-begin-regexp
      (lambda () (or (wisi-in-string-or-comment-p)
@@ -1478,8 +1520,8 @@ Prompts with completion, defaults to filename at point."
 
 ;;;; ada-mode
 
-(defvar which-func-functions nil) ;; which-func.el
-(defvar which-func-non-auto-modes nil) ;; ""
+(defvar which-func-functions) ;; which-func.el
+(defvar which-func-non-auto-modes) ;; ""
 
 ;;;###autoload
 (define-derived-mode ada-mode prog-mode "Ada"
@@ -1536,7 +1578,8 @@ Prompts with completion, defaults to filename at point."
   ;; loaded later, it will use the add-log which-function, which
   ;; forces a navigate parse.
   (add-hook 'which-func-functions #'ada-which-function nil t)
-  (add-to-list 'which-func-non-auto-modes 'ada-mode)
+  (when (boundp 'which-func-non-auto-modes)
+    (add-to-list 'which-func-non-auto-modes 'ada-mode))
 
   ;;  Support for align
   (add-to-list 'align-dq-string-modes 'ada-mode)
@@ -1559,8 +1602,6 @@ Prompts with completion, defaults to filename at point."
 
   (setq align-mode-rules-list ada-align-rules)
 
-  (easy-menu-add ada-mode-menu ada-mode-map)
-
   (wisi-setup
    :indent-calculate '(ada-wisi-comment)
    :post-indent-fail 'ada-wisi-post-parse-fail
@@ -1637,6 +1678,9 @@ Prompts with completion, defaults to filename at point."
 
 ;;;;; Global initializations
 
+;;;###autoload
+(add-to-list 'auto-mode-alist '("\\.ad[abs]\\'" . ada-mode))
+
 (when (featurep 'imenu)
   (require 'ada-imenu))
 
