@@ -19,7 +19,12 @@ fi
 # support for libadalang is still experimental
 gnatprep  -DHAVE_LIBADALANG="no" -DELPA="yes" -DHAVE_GNAT_UTIL=$HAVE_GNAT_UTIL ada_mode_wisi_parse.gpr.gp ada_mode_wisi_parse.gpr
 
-WISI_DIR="../wisi-3.1.0"
+if [ -d ../wisi-3.1.? ]; then
+    WISI_DIR=`ls -d ../wisi-3.1.?`
+else
+    # try devel version
+    WISI_DIR=`ls -d ../wisi-3.1.?.0.*`
+fi
 
 gnatprep -DELPA="yes" $WISI_DIR/wisi.gpr.gp $WISI_DIR/wisi.gpr
 
@@ -30,7 +35,7 @@ gnatprep -DELPA="yes" $WISI_DIR/wisi.gpr.gp $WISI_DIR/wisi.gpr
 #  - Run gprclean, to allow changing compilers and other drastic things
 #  - Don't delete ada_lr1_parse_table.txt
 
-gprclean -r -P ada_mode_wisi_parse.gpr -aP$WISI_DIR
+gprclean -q -r -P ada_mode_wisi_parse.gpr -aP$WISI_DIR
 
 gprbuild -p -j8 -P ada_mode_wisi_parse.gpr -aP $WISI_DIR "$@"
 
