@@ -3,7 +3,7 @@
 ;; gpr-query supports Ada and any gcc language that supports the
 ;; AdaCore -fdump-xref switch (which includes C, C++).
 ;;
-;; Copyright (C) 2013 - 2020  Free Software Foundation, Inc.
+;; Copyright (C) 2013 - 2021  Free Software Foundation, Inc.
 
 ;; Author: Stephen Leake <stephen_leake@member.fsf.org>
 ;; Maintainer: Stephen Leake <stephen_leake@member.fsf.org>
@@ -302,7 +302,7 @@ Must match gpr_query.adb Version.")
     ))
 
 (defun gpr-query-session-wait (session command-type)
-  "Wait for the current COMMAND-TYPE (one of 'xref or 'symbols) command to complete."
+  "Wait for the current COMMAND-TYPE (one of 'xref or 'symbols) to complete."
   (when (and
 	 (eq command-type 'symbols)
 	 (null (gpr-query--session-symbols-process session)))
@@ -720,10 +720,21 @@ FILE is from gpr-query."
 
 (defun gpr-query-tree-refs (project item op)
   "Run gpr_query tree command OP on ITEM (an xref-item), return list of xref-items."
-  (with-slots (summary location) item
-    ;; 'location' may have line, column nil
-    (let ((eieio-skip-typecheck t))
-      (with-slots (file line column) location
+  ;; WORKAROUND: xref 1.3.2 xref-location changed from defclass to cl-defstruct
+  (with-no-warnings ;; "unknown slot"
+    (let ((summary (if (functionp 'xref-item-summary) (xref-item-summary item) (oref item :summary)))
+	  (location (if (functionp 'xref-item-location) (xref-item-location item) (oref item :location)))
+	  (eieio-skip-typecheck t)) ;; 'location' may have line, column nil
+      (let ((file (if (functionp 'xref-file-location-file)
+		      (xref-file-location-file location)
+		    (oref location :file)))
+	    (line (if (functionp 'xref-file-location-line)
+		      (xref-file-location-line location)
+		    (oref location :line)))
+	    (column (if (functionp 'xref-file-location-column)
+			(xref-file-location-column location)
+		      (oref location :column))))
+
 	(when (eq ?\" (aref summary 0))
 	  ;; gpr_query wants the quotes stripped
 	  (when column (setq column (+ 1 column)))
@@ -808,6 +819,9 @@ FILE is from gpr-query."
       (gpr-query-session-wait session 'symbols));; ensure symbol-locs is ready
     (gpr-query--session-symbol-locs session)))
 
+(cl-defmethod wisi-xref-completion-delim-regex ((_xref gpr-query-xref))
+  (concat "[_(.<>*]"))
+
 (cl-defmethod wisi-xref-completion-regexp ((_xref gpr-query-xref))
   gpr-query-completion-regexp)
 
